unit Unit1;

{$mode objfpc}{$H+}


// from https://forum.lazarus.freepascal.org/index.php/topic,35850.0.html


interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  OpenGLContext, GL;

type

  { TForm1 }

  TForm1 = class(TForm)
    Image1: TImage;
    OpenGLControl1: TOpenGLControl;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure OpenGLControl1Paint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FontBitmap: TBitmap;
    FontSize: integer;
    function CreateFontBitmap: TBitmap;
    procedure DisplayChar(C: Char);
    procedure DisplayString(S: String);
    function LoadFontBitmap: TBitmap;
  public
    { public declarations }
  end;


var
  Form1: TForm1;
  RectSize: Real = 1;
  RectPosX: Real = 0;
  RectPosY: Real = 0;
  RectDirX: Real = 0.02;
  RectDirY: Real = 0.02;


implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  FontSize:= 30;

  //FontBitmap:= LoadFontBitmap;
  FontBitmap:= CreateFontBitmap;

  Image1.Width:= FontBitmap.Width;
  Image1.Height:= FontBitmap.Height;
  Image1.Picture.Bitmap.Assign( FontBitmap);
end;

procedure TForm1.OpenGLControl1Paint(Sender: TObject);
begin
  // GL clear screen
  glLoadIdentity;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  //glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  //glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glDisable(GL_TEXTURE_2D);
  glClearColor(0, 0, 0, 1);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // GL preparation
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glTexImage2D(GL_TEXTURE_2D, 0, 4, FontBitmap.Width, FontBitmap.Height, 0,
    GL_RGB, GL_UNSIGNED_BYTE, FontBitmap.RawImage.Data);

  // Draw the text
  glTranslatef( -0.9, 0.8, 0);
  DisplayString( 'X' +IntToStr( Mouse.CursorPos.X));
  glTranslatef( 0.2, 0, 0);
  DisplayString( 'Y' +IntToStr( Mouse.CursorPos.Y));

  // Draw the rectangle
  glLoadIdentity;
  glTranslatef(RectPosX, RectPosY, 0.1);
  glScalef(RectSize, RectSize, RectSize);
  glBegin(GL_QUADS);
  glColor3ub(0, 255, 0);
  glVertex3f(-0.5, -0.5, 0);
  glVertex3f(0.5, -0.5, 0);
  glVertex3f(0.5, 0.5, 0);
  glVertex3f(-0.5, 0.5, 0);
  glEnd();

  OpenGLControl1.SwapBuffers;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  R: Real;
begin
  R := Random/20 - 1/40;
  RectSize := RectSize + R;
  if (RectSize > 1) then RectSize := 1;
  if (RectSize < 0.1) then RectSize := 0.1;

  RectPosX := RectPosX + RectDirX;
  RectPosY := RectPosY + RectDirY;
  if (RectPosX > 0.8) or (RectPosX < -0.8) then RectDirX := -RectDirX;
  if (RectPosY > 0.8) or (RectPosY < -0.8) then RectDirY := -RectDirY;
  if (Random < 0.1) then RectDirX := -RectDirX;
  if (Random < 0.1) then RectDirY := -RectDirY;

  OpenGLControl1.Invalidate;  // OpenGL does not "repaint" without Invalidate here !
end;

procedure TForm1.DisplayChar(C: Char);
const
  CharWidth  = 0.08;
  CharHeight = 0.1;
var
  FontPos    : Byte;
  FontWidth  : real;
  FontHeight : real;
  X, Y       : real;
begin
  FontPos := 0;
  //if (C >= 'a') and (C <= 'z') then FontPos := ord(C) - 96;    // ASCII a is 97
  //if (C >= 'A') and (C <= 'Z') then FontPos := ord(C) - 64+26; // ASCII A is 65
  //if (C >= '0') and (C <= '9') then FontPos := ord(C) - 47+52; // ASCII 0 is 48

  // missing here:  (C >= 'a') and (C <= 'z') ..
  if (C >= 'A') and (C <= 'Z') then FontPos := ord(C) - 29;
  if (C >= '0') and (C <= '9') then FontPos := ord(C) - 29;
  if (FontPos = 0) then Exit;

  //FontWidth  := 30/FontBitmap.Width;  // 30 is the width of the font
  //FontHeight := 30/FontBitmap.Height; // 30 is the height of the font
  FontWidth  := FontSize / FontBitmap.Width;  // FontSize is the width of the font
  FontHeight := FontSize / FontBitmap.Height; // FontSize is the height of the font
  X := ((FontPos-1) mod 10) * FontWidth;
  Y := ((FontPos-1) div 10) * FontHeight;

  glBegin( GL_QUADS);
  glColor3ub( 255, 255, 255);
  glTexCoord2f( X, Y+FontHeight);
  glVertex3f( 0, 0, 0);
  glTexCoord2f( X+FontWidth, Y+FontHeight);
  glVertex3f( CharWidth, 0, 0);
  glTexCoord2f( X+FontWidth, Y);
  glVertex3f( CharWidth, CharHeight, 0);
  glTexCoord2f( X, Y);
  glVertex3f( 0, CharHeight, 0);
  glEnd();
end;

procedure TForm1.DisplayString(S: String);
const
  CharWidth  = 0.08;
var
  i: Integer;
begin
  for i := 1 to Length(S) do begin
    DisplayChar( S[i]);
    glTranslatef( CharWidth, 0, 0);
  end;
end;

function TForm1.LoadFontBitmap: TBitmap;
const
  FontBitmapFile = 'FontTexture.bmp';   // chars of 30 x 30 pixel
begin
  result:= TBitmap.Create;
  result.LoadFromFile( FontBitmapFile);
end;

// from https://www.lazarusforum.de/viewtopic.php?p=141767#p141767
function TForm1.CreateFontBitmap: TBitmap;
const
  //size = 32;
  //size = 30;
  //FontCount = 94;
  FontCount = 99;
var
  i: integer;
begin
  Result := TBitmap.Create;
  //Result.Width := FontSize * FontCount;
  //Result.Height := FontSize;
  Result.Width := FontSize * 10;
  Result.Height := FontSize * 10;

  Result.Canvas.FloodFill( 0, 0, clWhite, fsBorder);

  Result.Canvas.Font.Color  := clRed;
  //Result.Canvas.Font.Color  := clBlack;
  //Result.Canvas.Font.Color  := clWhite;
  Result.Canvas.Font.Name   := 'monospace';
  //Result.Canvas.Font.Height := size div 2;
  Result.Canvas.Font.Height := FontSize;
  for i := 0 to FontCount - 1 do begin
    //Result.Canvas.TextOut(I * FontSize, 0, char(i + size));
    Result.Canvas.TextOut( (i mod 10) * FontSize, (i div 10) * FontSize, char(i + FontSize));
  end;
end;


end.
